﻿using PeacePDS.Data;
using PeacePDS.Models.Entities;
using PeacePDS.Models.ViewModels;
using Microsoft.EntityFrameworkCore;
using static PeacePDS.Responses.CustomResponses;
using PeacePDS.Utils;

namespace PeacePDS.Services
{
    public class UserService : IUserService
    {
        private AppDbContext _appDbContext;

        public UserService(AppDbContext appDbContext)
        {
            _appDbContext = appDbContext;
        }

        public async Task<LoginResponse> LoginAsync(LoginViewModel model)
        {
            if (string.IsNullOrWhiteSpace(model.UserName) || string.IsNullOrWhiteSpace(model.Password))
                return new LoginResponse(false, "Invalid account information");

            var user = _appDbContext.Users.Where(u => u.Name == model.UserName).FirstOrDefault();
            if (user is null || !PasswordHasher.VerifyPassword(model.Password, user.HashedPassword))
                return new LoginResponse(false, "Invalid account information");

            user = _appDbContext.Users.Where(u => u.Name == model.UserName).Include(u => u.UserPolicies).FirstOrDefault();

            return new LoginResponse(true, "Success", user);
        }

        public async Task<RegistrationResponse> RegisterAsync(RegisterViewModel model)
        {
            if (string.IsNullOrWhiteSpace(model.UserName) || string.IsNullOrWhiteSpace(model.Password) || string.IsNullOrEmpty(model.ConfirmPassword) || !string.Equals(model.Password, model.ConfirmPassword))
                return new RegistrationResponse(false, "Invalid account information");

            var user = _appDbContext.Users.Where(u => u.Name == model.UserName).FirstOrDefault();
            if (user is not null)
                return new RegistrationResponse(false, "Unavailable account name");
            
            User newUser = new User()
            {
                Name = model.UserName,
                HashedPassword = PasswordHasher.HashPassword(model.Password),
            };

            _appDbContext.Users.Add(newUser);
            await _appDbContext.SaveChangesAsync();

            return new RegistrationResponse(true, "Success");
        }
    }
}
